#!/usr/bin/python -t
# -*- mode: Python; indent-tabs-mode: nil; -*-
#
# List bugzilla component descriptions page via web
# Author: Michael Schwendt <mschwendt@fedoraproject.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import urllib
from HTMLParser import HTMLParser
from htmlentitydefs import entitydefs
import ConfigParser

reventitydict = dict([(v,k) for k,v in entitydefs.items()])


class BugzillaLister(HTMLParser):

    def __init__(self, bzvars):
        self._bzvars = bzvars
        HTMLParser.__init__(self)

    def reset(self):
        HTMLParser.reset(self)
        self.state = 'start'
        self.datastate = None
        self.starttag = None
        self.attrs = []
        self.compdict = {}

    def handle_starttag(self, tag, attrs):
        self.starttag = tag
        self.attrs = attrs
        func = getattr(self, 'state_%s' % self.state)
        func()

    def handle_endtag(self, tag):
        self.endtag = tag

    def handle_data(self, data):
        if not self.datastate:
            return
        func = getattr(self, 'datastate_%s' % self.datastate)
        func(data)

    def handle_charref(self, name):
        if name == '64':
            return '@'
        key = '&#%s;' % name
        if reventitydict.has_key(key):
            return reventitydict[key]
        else:
            return name

    # TODO:
    # livna bugzilla returns package descriptions like "... rip&burn ..."
    # where &burn is recognised as an entity
    def handle_entityref(self, name):
#        print name
        pass

    def replaceentities(self, data):
        while True:
            p1 = data.find('&')
            if p1<0:
                return data
            p2 = data[p1:].find(';')
            if p2<0:
                return data
            e = data[p1:p1+p2+1]
            if e == '&#64;':
                e = '@'
            else:
                if reventitydict.has_key(e):
                    e = reventitydict[e]
            
            data = data[0:p1]+e+data[p1+p2+1:]

    def state_start(self):
        if self.starttag == 'a':
            for (n,v) in self.attrs:
                if n == 'name':
                    self.compdict.setdefault(v,{})
                    self.currentname = v
                    self.state = 'getowner'

    def state_getowner(self):
        if self.starttag == 'a':
            for (n,v) in self.attrs:
                if n == 'href' and v.startswith('mailto:'):
                    self.compdict[self.currentname]['owner'] = self.replaceentities(v[7:])
                    # TODO: unencode
                    if self._bzvars['getqacontact'] == 'True':
                        self.state = 'getqacontact'
                    else:
                        self.compdict[self.currentname]['qacontact'] = ''
                        self.state = 'getdescription'

    def state_getqacontact(self):
        if self.starttag == 'a':
            for (n,v) in self.attrs:
                if n == 'href' and v.startswith('mailto:'):
                    self.compdict[self.currentname]['qacontact'] = self.replaceentities(v[7:])
                    # TODO: unencode
                    self.state = 'getdescription'

    def state_getdescription(self):
        if self.starttag == 'td':
            self.datastate = 'copydescription'
            self.state = 'start'

    def datastate_copydescription(self, data):
        if self.compdict[self.currentname].has_key('description'):
            self.compdict[self.currentname]['description'] += data.rstrip()
        else:
            self.compdict[self.currentname]['description'] = data.rstrip()
#        print data
        self.datastate = None


def getBugzillaComps(bzvars,product):
    fields = {}
    fields['product'] = product
    bzargs = urllib.urlencode(fields)
    f = urllib.urlopen(bzvars['compsurl'], bzargs)
    desccomppage = f.read()
    bzl = BugzillaLister(bzvars)
    bzl.feed(desccomppage)
    del desccomppage
    return bzl.compdict


### testing only ###

if __name__ == "__main__":
    import errno, os, sys
    from optparse import OptionParser

    usage = "Usage: %s [options...]" % sys.argv[0]
    parser = OptionParser(usage=usage)
    parser.add_option("-c", "--config", default=None,
                      help='config file to use')
    parser.add_option("-p", "--product", default=[], action='append',
                      help="product section id to use")
    (opts, args) = parser.parse_args()

    if not opts.config or not len(opts.product)>0:
        print >> sys.stderr, "Invalid options!"
        sys.exit(errno.EINVAL)

    cp = ConfigParser.ConfigParser()
    cp.read(opts.config)
    bzvars = {}
    for (k,v) in cp.items('bugzilla'):
       bzvars[k] = v

    for p in opts.product:
        product = cp.get(p,'product')
        compdict = getBugzillaComps(bzvars,product)
        comps = compdict.keys()
        comps.sort()
        for name in comps:
            owner = compdict[name]['owner']
            description = compdict[name]['description']
            qacontact = compdict[name]['qacontact']
            print '%s|%s|%s|%s|%s|' % (product,name,description,owner,qacontact)
    sys.exit(0)
